"""
Updates the game version for export (based on the file .version).
"""

import re
import shutil
from argparse import ArgumentParser
from datetime import datetime
from pathlib import Path
from tempfile import mkstemp

from packaging import version


def get_today() -> datetime:
    return datetime.today()


def sed(pattern: str | re.Pattern[str],
        replace: str,
        source: Path,
        output: Path | None = None) -> None:
    """
    Read a source file, and for each line, replaces the pattern inplace.

    :param pattern: pattern to match
    :param replace: replacement str
    :param source: input filename
    :param output: output filename, if it's None replace the source in-place
    """
    _fd, name = mkstemp()
    with open(source, 'r') as fin, open(name, 'w') as fout:

        for line in fin:
            out = re.sub(pattern, replace, line)
            print(out)
            fout.write(out)

        fout.writelines(fin.readlines())

    if output is None:
        output = source

    shutil.move(name, output)


def read_version_file(source: Path) -> str:
    """
    Read a source file, and return it's first line.

    :param source: input filename
    :raises ValueError: raises an exception when the source is empty
    """
    with open(source, 'r') as fin:
        for line in fin:
            return line

    raise ValueError(f'{source} is an empty file')


def replace_version(game_version: version.Version,
                    input_cfg_file: Path,
                    output_cfg_file: Path | None = None) -> None:
    today = get_today().strftime('%Y%m%d')

    if output_cfg_file is None:
        output_cfg_file = input_cfg_file

    short_version = f"{game_version.major}.{game_version.minor}"
    release_version = f"{short_version}.{game_version.micro}"

    sed('application/file_version=.*$',
        f'application/file_version="{release_version}.{today}"',
        input_cfg_file, output_cfg_file)
    sed('application/product_version=.*$',
        f'application/product_version="{release_version}.{today}"',
        output_cfg_file)
    sed('application/version=.*$', f'application/version="{release_version}"',
        output_cfg_file)
    sed('application/short_version=.*$',
        f'application/short_version="{short_version}"', output_cfg_file)


def main() -> None:
    parser = ArgumentParser(description='Updates the game version for export.')
    parser.add_argument('--version', type=version.Version)
    parser.add_argument('--version-file', default='.version', type=Path)
    parser.add_argument('--cfg-file', default='export_presets.cfg', type=Path)

    options = parser.parse_args()

    game_version = version.parse(
        read_version_file(options.version_file) if options.version is
        None else options.version)

    replace_version(game_version, options.cfg_file)


if __name__ == "__main__":
    main()
